/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import ZEN_IDL.ast.*;
import ZEN_IDL.be.symbolTableUtils.*;
import ZEN_IDL.be.helperUtils.*;
import util.*;

import java.io.*;
import java.util.*;

/*********************************************************
* Generates stubs and skeleton inline marshalling and 
* unmarshalling code with forcing of endianessd.
*
* @author Olav Haugan
*
* 14.10.01
*********************************************************/
public class MQCInlineFEMarshalHelper extends MQCAbstractMarshalHelper
{
	private boolean floatDef; /* Float variable declared or not */
	private boolean doubleDef; /* Double variable declared or not */
	private	boolean longDef; /* long variables declared or not */			
	
	private	boolean bigEndian; /* long variables declared or not */			
		
	/*********************************************************
	* Constructor. Initializes variables.
	* 
	* @param Hashtable A method mapping object
	*        
	*********************************************************/
	public MQCInlineFEMarshalHelper(MQCConfig config, boolean bigEndian )
	{
		super(config);
		this.bigEndian = bigEndian;
	}

	/*********************************************************
	* Initializes the marshaller.
	* These variables are used to keep track of declared
	* variables so that we don't declare the same variable
	* twice.
	*********************************************************/
	public void init()
	{
		floatDef = false;
		doubleDef = false;
		longDef = false;		
	}

	/*********************************************************
	* Prints the marshalling code for 
	* either the client or server. 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	* @param boolean true if code is for client, false
  *                if for server.
	*
	* @return int The size of buffer needed.
	*********************************************************/
	public int printMarshal( MQCPrinter pw, InterfaceMethod iMethod, boolean Client )
	{
		if(Client)
			return printClientMarshal(pw, iMethod );
		else
			return printServerMarshal(pw, iMethod );
	}

	/*********************************************************
	* Prints the marshalling code for 
	* either the client or server. 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	* @param boolean true if code is for client, false
  *                if for server.
	*
	*********************************************************/
	public void printUnMarshal( MQCPrinter pw, InterfaceMethod iMethod, boolean Client )
	{
		if(Client)
			printClientUnMarshal(pw, iMethod );
		else
			printServerUnMarshal(pw, iMethod );
	}
	
	/*********************************************************
	* Prints the inline client marshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	public int printClientMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		boolean padding = config.isPadding();
		
		e = iMethod.paramVector.elements();									
		InterfaceMethodParam arg = null;
		
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("out") )
			{
				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					if(padding)
					{
						printAlign(pw, arg.paramType, true);
					}
					printBufferInc(pw, arg.paramType, true);
					if(arg.paramType.equals("float"))
						doMarshalFloat(pw, arg);
					else if(arg.paramType.equals("double"))
						doMarshalDouble(pw, arg);
					else if(arg.paramType.equals("boolean"))
						doMarshalBoolean(pw, arg);
					else
						doMarshal(pw, arg);
				}
			}
		}
		return 0;
	}


	/*********************************************************
	* Prints the inline marshalling code for basic
	* types.
	*
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doMarshal(MQCPrinter pw, InterfaceMethodParam arg )
	{
		int paramSize = codeGenHelper.getParamSize(arg.paramType);
		if(bigEndian)
		{		
			for(int i = paramSize-1; i > 0; i--)
			{
				pw.print("_msg[ptr.value++] = (byte) ");
				if(!arg.paramInOut.equals("in"))
					pw.nprintln("(" + arg.paramName + ".value >> " + (8 * i) + ");" );
				else
					pw.nprintln("(" + arg.paramName + " >> " + (8 * i) + ");" );
			}
			pw.print("_msg[ptr.value++] = (byte) ");
			if(!arg.paramInOut.equals("in"))
				pw.nprintln(arg.paramName + ".value;" );
			else
				pw.nprintln(arg.paramName + ";" );
		}
		else
		{		
			int i = 0;
			for(i = 0; i <  paramSize-1; i++)
			{
				pw.print("_msg[ptr.value++] = (byte) ");
				if(!arg.paramInOut.equals("in"))
					pw.nprintln("(" + arg.paramName + ".value >> " + (8 * i) + ");" );
				else
					pw.nprintln("(" + arg.paramName + " >> " + (8 * i) + ");" );
			}
			pw.print("_msg[ptr.value++] = (byte) ");
			if(!arg.paramInOut.equals("in"))
				pw.nprintln("(" + arg.paramName + ".value >> " + (8 * i) + ");" );
			else
				pw.nprintln("(" + arg.paramName + " >> " + (8 * i) + ");" );
		}
	}	

	/*********************************************************
	* Prints the inline server marshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	public int printServerMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg = null;
		boolean padding = config.isPadding();

		int paramSize = codeGenHelper.getParamSize(iMethod.returnType);
		
		if(padding)
		{
			printAlign(pw, iMethod.returnType, true );
		}
		printBufferInc(pw, iMethod.returnType, true );
		
		if(iMethod.returnType.equals("float"))
			doMarshalFloat(pw, null);
		else if(iMethod.returnType.equals("double"))
			doMarshalDouble(pw, null);
		else if(iMethod.returnType.equals("boolean"))
			doMarshalBoolean(pw, null);
		else if(!iMethod.returnType.equals("void"))
		{
			if(bigEndian)
			{
				for(int i = paramSize-1; i > 0; i--)
				{
					pw.print("_msg[ptr.value++] = (byte) ");
					pw.nprintln("(_result >> " + (8 * i) + ");" );
				}
				pw.println("_msg[ptr.value++] = (byte) _result;");
			}
			else
			{
				int i = 0;
				for(i = 0; i < paramSize-1; i++)
				{
					pw.print("_msg[ptr.value++] = (byte) ");
					pw.nprintln("(_result >> " + (8 * i) + ");" );
				}
				pw.println("_msg[ptr.value++] = (byte) (_result >> " + (8 * i) + ");" );
			}
		}
		e = iMethod.paramVector.elements();
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("in") )
			{
				if(padding)
				{
					printAlign(pw, arg.paramType, true );
				}
				
				printBufferInc(pw, arg.paramType, true );
				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					if(arg.paramType.equals("float"))
						doMarshalFloat(pw, arg);
					else if(arg.paramType.equals("double"))
						doMarshalDouble(pw, arg);
					else if(arg.paramType.equals("boolean"))
						doMarshalBoolean(pw, arg);
					else
						doMarshal(pw, arg);					
				}
			}
		}
		return 0;
	}


	/*********************************************************
	* Prints the inline client unmarshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	public void printClientUnMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg;
		boolean padding = config.isPadding();
		
		int paramSize;
		
		if(!iMethod.returnType.equals("void"))
		{
			if(codeGenHelper.isPrimitiveType(iMethod.returnType))
			{
				paramSize = codeGenHelper.getParamSize(iMethod.returnType);
				if(padding)
				{
					printAlign(pw, iMethod.returnType, true);
				}
				if(iMethod.returnType.equals("float"))
					doUnMarshalFloat(pw, null);
				else if(iMethod.returnType.equals("double"))
					doUnMarshalDouble(pw, null);
				else if(iMethod.returnType.equals("boolean"))
					doUnMarshalBoolean(pw, null );
				else if(iMethod.returnType.equals("long long") || iMethod.returnType.equals("unsigned long long"))
					doUnMarshalLong(pw, null );
				else
				{
					if(bigEndian)
					{
						pw.print("_result = (" + codeGenHelper.mapIDL2Java( iMethod.returnType ) + ") (");
						for(int i = paramSize-1; i > 0; i--)
						{
							pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
						}
						pw.nprintln("(_msg[ptr.value++] & 0xFF));");
					}
					else
					{
						pw.print("_result = (" + codeGenHelper.mapIDL2Java( iMethod.returnType ) + ") (");
						for(int i = 0; i < paramSize-1; i++)
						{
							pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
						}
						pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * (paramSize-1)) + "));");
					}
				}
			}
		}
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("in") )
			{
				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					paramSize = codeGenHelper.getParamSize(arg.paramType);
					if(padding)
					{
						printAlign(pw, arg.paramType, true);
					}
					if(arg.paramType.equals("float"))
						doUnMarshalFloat(pw, arg);
					else if(arg.paramType.equals("double"))
						doUnMarshalDouble(pw, arg);
					else if(arg.paramType.equals("boolean"))
						doUnMarshalBoolean(pw, arg);
					else if(arg.paramType.equals("long long") || arg.paramType.equals("unsigned long long"))
						doUnMarshalLong(pw, arg);
					else
						doUnMarshal(pw, arg);
				}
			}
		} //while
	}

	/*********************************************************
	* Prints the server inline unmarshalling code
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	*********************************************************/
	public void printServerUnMarshal(MQCPrinter pw, InterfaceMethod iMethod )
	{
		Enumeration e = iMethod.paramVector.elements();
		InterfaceMethodParam arg;
		boolean padding = config.isPadding();

		int paramSize = 0;
		
		while( e.hasMoreElements() )
		{
			arg = (InterfaceMethodParam) e.nextElement();
			if( !arg.paramInOut.equals("out") )
			{
				if(padding)
				{
					printAlign(pw, arg.paramType, true);
				}

				if(codeGenHelper.isPrimitiveType(arg.paramType))
				{
					if(!arg.paramInOut.equals("in"))
					{
						pw.print(codeGenHelper.mapIDL2JavaHolder( arg.paramType ) );
						pw.nprint( " " + arg.paramName + " = " );
						pw.nprintln("new " + codeGenHelper.mapIDL2JavaHolder( arg.paramType ) + "();" );
					}
					else
					{
						pw.print(codeGenHelper.mapIDL2Java( arg.paramType ) );
						pw.nprintln( " " + arg.paramName + ";" );
					}
					if(arg.paramType.equals("float"))
						doUnMarshalFloat(pw, arg);
					else if(arg.paramType.equals("double"))
						doUnMarshalDouble(pw, arg);
					else if(arg.paramType.equals("boolean"))
						doUnMarshalBoolean(pw, arg);
					else if(arg.paramType.equals("long long") || arg.paramType.equals("unsigned long long"))
						doUnMarshalLong(pw, arg);
					else
						doUnMarshal(pw, arg);
				}
			}
			else
			{
				pw.print(codeGenHelper.mapIDL2JavaHolder( arg.paramType ) );
				pw.nprint( " " + arg.paramName + " = " );				
				pw.nprintln("new " + codeGenHelper.mapIDL2JavaHolder( arg.paramType ) + "();" );
			}
		}
	}

	/*********************************************************
	* Prints the un-marshaling of basic types
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doUnMarshal(MQCPrinter pw, InterfaceMethodParam arg)
	{
		int paramSize = codeGenHelper.getParamSize(arg.paramType);

		if(bigEndian)
		{
			if(!arg.paramInOut.equals("in"))
				pw.print(arg.paramName + ".value = (" + codeGenHelper.mapIDL2Java( arg.paramType ) + ") (");
			else
				pw.print(arg.paramName + " = (" + codeGenHelper.mapIDL2Java( arg.paramType ) + ") (");

			for(int i = paramSize-1; i > 0; i--)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("(_msg[ptr.value++] & 0xFF));");
		}
		else
		{
			if(!arg.paramInOut.equals("in"))
				pw.print(arg.paramName + ".value = (" + codeGenHelper.mapIDL2Java( arg.paramType ) + ") (");
			else
				pw.print(arg.paramName + " = (" + codeGenHelper.mapIDL2Java( arg.paramType ) + ") (");
			for(int i = 0; i < paramSize-1; i++)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * (paramSize-1)) + "));");
		}
	}

	/*********************************************************
	* Prints the marshaling of boolean 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doMarshalBoolean(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(arg == null)
		{
			pw.println("_msg[ptr.value++] = (byte) (_result ? 1 : 0);");		
		}
		else
		{
			if(arg.paramInOut.equals("in"))
				pw.println("_msg[ptr.value++] = (byte) (" + arg.paramName + " ? 1 : 0);");
			else
				pw.println("_msg[ptr.value++] = (byte) (" + arg.paramName + ".value ? 1 : 0);");
		}
	}
	
	/*********************************************************
	* Prints the marshaling of float 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doMarshalFloat(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(!floatDef)
			pw.print("int fval = ");
		else
			pw.print("fval = ");
		floatDef = true;
		
		if(arg == null)
		{
			pw.nprintln("java.lang.Float.floatToIntBits(_result);");		
		}
		else
		{
			if(!arg.paramInOut.equals("in"))
				pw.nprintln("java.lang.Float.floatToIntBits(" + arg.paramName + ".value);");
			else
				pw.nprintln("java.lang.Float.floatToIntBits(" + arg.paramName + ");");		
		}
		int paramSize = codeGenHelper.getParamSize("float");
		if(bigEndian)
		{
			for(int i = paramSize-1; i > 0; i--)
			{
				pw.print("_msg[ptr.value++] = (byte) ");
				pw.nprintln("(fval >> " + (8 * i) + ");" );
			}
			pw.println("_msg[ptr.value++] = (byte) fval;");
		}
		else
		{
			for(int i = 0; i < paramSize-1; i++)
			{
				pw.print("_msg[ptr.value++] = (byte) ");
				pw.nprintln("(fval >> " + (8 * i) + ");" );
			}
			pw.println("_msg[ptr.value++] = (byte) (fval >> " + (8 * (paramSize-1)) + ");" );
		}
	}


	/*********************************************************
	* Prints the marshaling of double
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doMarshalDouble(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(!doubleDef)
			pw.print("long dval = ");
		else
			pw.print("dval = ");
		doubleDef = true;
		
		if(arg == null)
		{
			pw.nprintln("java.lang.Double.doubleToLongBits(_result);");
		}
		else
		{
			if(!arg.paramInOut.equals("in"))
				pw.nprintln("java.lang.Double.doubleToLongBits(" + arg.paramName + ".value);");
			else
				pw.nprintln("java.lang.Double.doubleToLongBits(" + arg.paramName + ");");
		}

		int paramSize = codeGenHelper.getParamSize("double");
		if(bigEndian)
		{
			for(int i = paramSize-1; i > 0; i--)
			{
				pw.print("_msg[ptr.value++] = (byte) ");
				pw.nprintln("(dval >> " + (8 * i) + ");" );
			}
			pw.println("_msg[ptr.value++] = (byte) dval;");
		}
		else
		{
			for(int i = 0; i < paramSize-1; i++)
			{
				pw.print("_msg[ptr.value++] = (byte) ");
				pw.nprintln("(dval >> " + (8 * i) + ");" );
			}
			pw.println("_msg[ptr.value++] = (byte) (dval >> " + (8 * (paramSize-1)) + ");" );
		}
	}

	/*********************************************************
	* Prints the un-marshaling of boolean 
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doUnMarshalBoolean(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(arg == null)
		{
			pw.println("_result = (_msg[ptr.value++] == 1) ? true : false;");		
		}
		else
		{
			if(arg.paramInOut.equals("in"))
				pw.println(arg.paramName + " = (_msg[ptr.value++] == 1) ? true : false;");
			else
				pw.println(arg.paramName + ".value = (_msg[ptr.value++] == 1) ? true : false;");
		}
	}

	/*********************************************************
	* Prints the un-marshaling of float
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doUnMarshalFloat(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(!floatDef)
			pw.println("int fval = 0;");
		floatDef = true;
		
		int paramSize = codeGenHelper.getParamSize("float");
		if(bigEndian)
		{
			pw.print("fval = (int) (" );
			for(int i = paramSize-1; i > 0; i--)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("(_msg[ptr.value++] & 0xFF));");
		}
		else
		{
			pw.print("fval = (int) (" );
			for(int i = 0; i < paramSize-1; i++)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * (paramSize-1)) + "));");
		}

		if(arg == null)
		{
			pw.println("_result = java.lang.Float.intBitsToFloat(fval);");
		}
		else
		{
			if(!arg.paramInOut.equals("in"))
				pw.println(arg.paramName + ".value = java.lang.Float.intBitsToFloat(fval);");
			else
				pw.println(arg.paramName + " = java.lang.Float.intBitsToFloat(fval);");
		}
	}

	/*********************************************************
	* Prints the un-marshaling of double
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doUnMarshalDouble(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(!doubleDef)
			pw.println("long dval = 0;");
		if(!longDef)
		{
			pw.println("long lval1 = 0;");
			pw.println("long lval2 = 0;");
		}
		longDef = true;
		doubleDef = true;		

		int paramSize = codeGenHelper.getParamSize("double");

		if(bigEndian)
		{
			pw.print("lval1 = (long) (" );
			for(int i = (paramSize/2)-1; i > 0; i--)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("(_msg[ptr.value++] & 0xFF));");
			pw.print("lval2 = (long) (" );
			for(int i = (paramSize/2)-1; i > 0; i--)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("(_msg[ptr.value++] & 0xFF));");
			pw.println("dval = (long) ((lval1 << 32) | (lval2 & 0xFFFFFFFFl));");	
		}
		else
		{
			pw.print("lval1 = (long) (" );
			for(int i = 0; i < (paramSize/2)-1; i++)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * ((paramSize/2)-1)) + "));");
			pw.print("lval2 = (long) (" );
			for(int i = 0; i < (paramSize/2)-1; i++)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * ((paramSize/2)-1)) + "));");
			pw.println("dval = (long) ((lval2 << 32) | (lval1 & 0xFFFFFFFFl));");
		}		

		if(arg == null)
		{
			pw.println("_result = java.lang.Double.longBitsToDouble(dval);");
		}
		else
		{
			if(!arg.paramInOut.equals("in"))
				pw.println(arg.paramName + ".value = java.lang.Double.longBitsToDouble(dval);");
			else
				pw.println(arg.paramName + " = java.lang.Double.longBitsToDouble(dval);");
		}
	}

	/*********************************************************
	* Prints the un-marshaling of long in a client
	* 
	* @param MQCPrinter The outputstream.
	*        
	* @param InterfaceMethodParam The method parameter
	*********************************************************/
	public void doUnMarshalLong(MQCPrinter pw, InterfaceMethodParam arg)
	{
		if(!longDef)
		{
			pw.println("long lval1 = 0;");
			pw.println("long lval2 = 0;");
		}
		longDef = true;
		
		int paramSize = codeGenHelper.getParamSize("long long");
		if(bigEndian)
		{
			pw.print("lval1 = (long) (" );
			for(int i = (paramSize/2)-1; i > 0; i--)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("(_msg[ptr.value++] & 0xFF));");
			pw.print("lval2 = (long) (" );
			for(int i = (paramSize/2)-1; i > 0; i--)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("(_msg[ptr.value++] & 0xFF));");
			if(arg == null)
			{
				pw.println("_result = (long) ((lval1 << 32) | (lval2 & 0xFFFFFFFFl));");
			}
			else
			{
				if(!arg.paramInOut.equals("in"))
					pw.println(arg.paramName + ".value = (long) ((lval1 << 32) | (lval2 & 0xFFFFFFFFl));");
				else
					pw.println(arg.paramName + " = (long) ((lval1 << 32) | (lval2 & 0xFFFFFFFFl));");
			}	
		}
		else
		{
			pw.print("lval1 = (long) (" );
			for(int i = 0; i < (paramSize/2)-1; i++)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");	
			}
			pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * ((paramSize/2)-1)) + "));");
			pw.print("lval2 = (long) (" );
			for(int i = 0; i < (paramSize/2)-1; i++)
			{
				pw.nprint("((_msg[ptr.value++] & 0xFF) << " + (8 * i) + ") | ");
			}
			pw.nprintln("((_msg[ptr.value++] & 0xFF) << " + (8 * ((paramSize/2)-1)) + "));");	

			if(arg == null)
			{
				pw.println("_result = (long) ((lval2 << 32) | (lval1 & 0xFFFFFFFFl));");
			}
			else
			{
				if(!arg.paramInOut.equals("in"))
					pw.println(arg.paramName + ".value = (long) ((lval2 << 32) | (lval1 & 0xFFFFFFFFl));");
				else
					pw.println(arg.paramName + " = (long) ((lval2 << 32) | (lval1 & 0xFFFFFFFFl));");
			}
		}
	}

	/*********************************************************
	* Prints alignment code
	* 
	* @param MQCPrinter The outputstream
	*
	* @param String parameter type
	*
	* @param boolean true if a holder object, false otherwise
	*********************************************************/
	public void printAlign(MQCPrinter pw, String type, boolean holder )
	{
		if(codeGenHelper.getAlignment(type) < 2)
			return;

		if(holder)
		{
			pw.print("ptr.value = (ptr.value + " + (codeGenHelper.getAlignment(type) - 1) + ") & ~" );
			pw.nprintln((codeGenHelper.getAlignment(type)-1) + ";");
		}
		else
		{
			pw.print("ptr = (ptr + " + (codeGenHelper.getAlignment(type) - 1) + ") & ~" );
			pw.nprintln((codeGenHelper.getAlignment(type)-1) + ";");
		}
	}
	
	/*********************************************************
	* Prints the buffer increase code
	* 
	* @param MQCPrinter The outputstream
	*
	* @param String parameter type
	*********************************************************/
	public void printBufferInc(MQCPrinter pw, String type, boolean client )
	{
		if(codeGenHelper.getParamSize(type) == 0)
			return;

		if(client)
		{
			pw.println("if(_msg.length < (ptr.value + " + codeGenHelper.getParamSize(type) + "))");
			pw.println("{");
			pw.println("\tbyte[] tmp = new byte[_msg.length + " + codeGenHelper.getParamSize(type) + " + 10];");
			pw.println("\tSystem.arraycopy(_msg, 0, tmp, 0, _msg.length);");
			pw.println("\t_msg = tmp;");
			pw.println("}");
		}
		else
		{
			pw.println("if(_output.length < (ptr.value + " + codeGenHelper.getParamSize(type) + "))");
			pw.println("{");
			pw.println("\tbyte[] tmp = new byte[_output.length + " + codeGenHelper.getParamSize(type) + " + 10];");
			pw.println("\tSystem.arraycopy(_output, 0, tmp, 0, _output.length);");
			pw.println("\t_output = tmp;");
			pw.println("}");
		}
	}
}
