/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import ZEN_IDL.ast.*;
import ZEN_IDL.be.symbolTableUtils.*;
import ZEN_IDL.be.helperUtils.*;
import util.*;

import java.io.*;
import java.util.*;


/*********************************************************
* MicroQoSCORBA visitor to "<interface>Operations.java"
* code. Some of the code is borrowed from the ZEN
* IDL compiler and changed.
*
* @author Olav Haugan
*
* 14.10.01
*********************************************************/
public class MQCInterfaceVisitor extends AbstractVisitor
{
	//The interface operations generator
	private MQCCodeGenInterface interfaceGen;

	//The constant code generator
	private MQCConstCodeGenerator constCodegenerator;

	/*********************************************************
	* Constructor. Initializes variables.
	* 
	* @param MQCConfig The configuration object
	*********************************************************/
	public MQCInterfaceVisitor(MQCCodeGenInterface interfaceGen)
	{
		this.interfaceGen = interfaceGen;
		constCodegenerator = new MQCConstCodeGenerator();
	}
	
	/*********************************************************
	* Visitor function for an IDL interface header. Called
	* by the visitor pattern for each interface header node.
	*
	* @param ASTinterface_header The interface hearder node
	*        
	* @param Object The data object
	*********************************************************/
	public Object visit(ASTinterface_header node, Object data)
	{
		VisitorState mState = (VisitorState) data;

		data = node.childrenAccept(this, data);
	
		// Generate methods in "<interfaceName>+Operations.java" file 
		interfaceGen.generateInterfaceOperations();
	// Generate "<interfaceName>+Helper.java" file 
		interfaceGen.generateInterfaceHelper();
	// Generate "<interfaceName>.java" file 
		interfaceGen.generateInterface();
		
		return data;
	}


	/*********************************************************
	* Visitor function for a IDL constant decl. Called
	* by the visitor pattern for each constant declared.
	*
	* @param ASTconst_dcl The const node from the parse tree
	*        
	* @param Object The data object
	*********************************************************/
	public Object visit(ASTconst_dcl node, Object data)
	{

		VisitorState mState = (VisitorState) data;

		constCodegenerator.setName( node.jjtGetChild(1).getFirstToken().toString() );
		constCodegenerator.setValue( node.jjtGetChild(2).getFirstToken().toString() );
		constCodegenerator.setType( getVariableType( (SimpleNode) node.jjtGetChild(0) ) );
		constCodegenerator.setInInterface( true );
		constCodegenerator.setPW( interfaceGen.getPW() );
		
		//Generate code for constants			
		constCodegenerator.generateCode();

		return data;
	}

	/*********************************************************
	* Construct the type of a variable from its node
	*
	* @param SimpleNode The variable node
	*        
	* @return String The variable type
	*********************************************************/
	private String getVariableType( SimpleNode node )
	{
		Token firstTypeToken = node.getFirstToken();
		Token lastTypeToken = node.getLastToken();

		String variableType = firstTypeToken.toString();

		while( firstTypeToken != lastTypeToken )
		{
			firstTypeToken = firstTypeToken.next;
			variableType = " " + firstTypeToken.toString();
		}
		return variableType;
	}

}
