/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package util;

import java.util.*;
import java.io.*;

/*********************************************************
* Class for keeping track of constants used in different
* classes.
*
* @author Olav Haugan
* @author Tarana Damania
* @author David McKinnon
*********************************************************/
public class Constants
{
  public static final String TOOL_TITLE = "MicroQoSCORBA Configuration Tool";
  

  //The names of the primitive types 
  public static final int NO_PRIMITIVE_TYPES  = 13;
  public static final String PRIMITIVE_TYPE[] = { 
    "boolean", "char", "octet", "short", "unsigned short", 
    "wchar", "long", "unsigned long", "long long", "unsigned long long", 
    "float", "double", "long double" };

  //The names of the complex types 
  public static final int NO_COMPLEX_TYPES  =  7;
  public static final String COMPLEX_TYPE[] = { 
    "struct", "union", "array", "sequence", "enum", "string", "wstring" }; 

  //The names of the exception types
  public static final int NO_EXCEPTION_TYPES  =  2;
  public static final String EXCEPTION_TYPE[] = { "system", "user" };

  public static final String CLIENT_HDR = "Client";
  public static final String SERVER_HDR = "Server";
  
  //The names of the transport types
  public static final int NO_TRANSPORTS       =  6;
  public static final String TRANSPORT_TYPE[] = { 
    "TCP/IP", "Unreliable UDP", "Go Back N UDP", "Stop-N-Wait UDP", 
    "Serial", "1-Wire" };
  
  //The names of the protocol types
  public static final int NO_PROTOCOLS       =  3;
  public static final String PROTOCOL_TYPE[] = { "GIOP", "GIOP-Lite", "MQC-IOP" };

  //The names of the endianess types
  public static final String ENDIANESS_TYPE[] = { "Big endian", "Little endian" };
  public static final String ENDIANESS = "Endianess";

  //Property and button names
  public static final String MSG_PAYLOAD      = "Message payload";
  public static final String NO_INTERFACES    = "Max # of interfaces";
  public static final String NO_METHODS       = "Max # of methods";
  public static final String HOMOGENEOUS_HW   = "Homogeneous HW";
  public static final String HETEROGENEOUS_HW = "Heterogeneous HW";
  public static final String FORCE_ENDIANESS  = "Force endianess";
  public static final String COMPRESS_NAMES   = "Compress Interface/method names";

  public static final String FORCE_CLDC   = "Force CLDC";
  public static final String DEBUG        = "Debug";
  public static final String DEBUG_LEVEL  = "Debug Level";
  public static final String DEBUG_MEMORY = "Debug Memory";
  public static final String DEBUG_TIMING = "Debug Timing";

  
  public static final String TR   = "Transport";  
  public static final String GIOP = "GIOP";  
  
  // Security Cipher constants
  public static final String SECURITY               = "Security--Confidentiality";
  public static final String SEC_CIPHER             = "Sec.Cipher";
  public static final String SEC_CIPHERS[]          = { "Null", "Caesar", "CAST5", "DES", "TripleDES", 
                                                       "IDEA", "MARS", "RC2", "RC4", "Rijndael", 
                                                       "Serpent", "SKIPJACK", "Square", "Twofish"};
  public static final String SEC_CIPHER_MODE        = "Mode";
  public static final String SEC_CIPHER_MODES[]     = { "ECB", "CBC" };
  public static final String SEC_CIPHER_PADDING     = "Padding";
  public static final String SEC_CIPHER_PADDINGS[]  = { "NONE", "PKCS5" };
  public static final String SEC_CIPHER_KEY_LEN     = "Cipher.Key.Length";
  public static final String SEC_CIPHER_KEY_DEFS[] = { "8", "8", "128", "56", "168", "128", "128", 
                                                       "128", "128", "128", "128", "80", "128", "128" };
  public static final String SEC_CIPHER_KEY_HINTS[] = { "Null - 8 bits", "Caesar - 8 bits", 
                                                        "CAST5 - 40,48,...,128 bits", "DES - 56 bits", 
                                                        "TripleDES - 168 bits", "IDEA - 128 bits", 
                                                        "MARS - 128,192,256 bits", "RC2 - 16,24,...,1024 bits", 
                                                        "RC4 - 8,16,...,2048 bits", "Rijndael - 128,192,256 bits", 
                                                        "Serpent - 128,192,256 bits",
                                                        "SKIPJACK - 80 bits", "Square - 128 bits", 
                                                        "Twofish - 128,192,256 bits" };
                                                        
  public static final String SECURITY_MD   = "Security--Message Digests";
  public static final String SEC_MD_LABEL  = "Sec.MD";
  public static final String SEC_MD_ALGS[] = { "Null", "CRC32", "MD2", "MD4", "MD5", "RIPEMD", "RIPEMD128",
                                               "RIPEMD160", "SHA0", "SHA1", "SHA256", "SHA384",
                                               "SHA512", "Tiger" };
  public static final String SEC_HMAC_KEY_HINTS[] = {  "Null - 8 bits", "CRC32 - 32 bits", "MD2 - 128 bits", "MD4 - 512 bits", 
                                                       "MD5 - 512 bits", "RIPEMD - 512 bits", "RIPEMD128 - 512 bits",
                                                       "RIPEMD160 - 512 bits", "SHA0 - 512 bits", "SHA1 - 512 bits", 
                                                       "SHA256 - 1024 bits", "SHA384 - 1024 bits",
                                                       "SHA512 - 1024 bits", "Tiger - 512 bits"};

  public static final String SECURITY_MAC        = "Security--Message Authentication Codes";
  public static final String SEC_MAC_LABEL       = "Sec.MAC";
  public static final String SEC_MAC_ALGS[]      = { "Null", "HMAC" };
  public static final String SEC_MAC_KEY_LEN     = "MAC.Key.Length";
  public static final String SEC_MAC_KEY_DEFS[]  = { "8", "512"};
  public static final String SEC_MAC_KEY_HINTS[] = { "Null 8 bits", "HMAC use MD hint" };

  public static final String SECURITY_AUDIT       = "Security--Audit";
  public static final String SEC_AUDIT_LABEL      = "Sec.Audit";
  public static final String SEC_AUDIT_ALGS[]     = { "Null", "Low", "Medium", "High" };
  public static final String SEC_AUDIT_LOG_LEN    = "Audit.Log.Length";
  public static final String SEC_AUDIT_LOG_DEFS[] = { "0", "25", "50", "100" };
  
  public static final String REALTIME = "Realtime";
  public static final String RT_SOFT  = "Realtime_Soft";  
  public static final String RT_HARD  = "Realtime_Hard";  

  public static final String NETWORK               = "Network QoS";
  public static final String NET_GUARANTEED_BW     = "Guaranteed BW";  
  public static final String NET_GUARANTEED_BW_NUM = "Guaranteed BW num";  
  public static final String NET_LOW_LATENCY       = "Low lantecy";  
  
  
  public static final String INLINE_MARSHAL = "Use inline marshaling/un-marshaling (in a proxy)";
  public static final String LIB_MARSHAL = "Use marshaling/un-marshaling library (in ORB library)";
  public static final String SYS_EXCEPTIONS  = "System_Exceptions";
  public static final String USER_EXCEPTIONS = "User_Exceptions";

  public static final String TR_SERIAL  = "Serial";  
  public static final String TR_1WIRE   = "1-Wire";  
  public static final String TR_TCP     = "TCP/IP";  
  //public static final String TR_UDP   = "UDP";
  public static final String TR_UNRELIABLE_UDP  = "Unreliable UDP";
  public static final String TR_GOBACKN_UDP     = "Go Back N UDP";
  public static final String TR_STOPANDWAIT_UDP = "Stop-N-Wait UDP";
  

  public static final String TR_SERIAL_STR = "Serial";  
  public static final String TR_1WIRE_STR  = "OneWire";  
  public static final String TR_TCP_STR    = "TCP";  
  //public static final String TR_UDP_STR  = "UDP";
  public static final String TR_UNRELIABLE_UDP_STR  = "UDP";
  public static final String TR_GOBACKN_UDP_STR     = "UDPGoBackN";
  public static final String TR_STOPANDWAIT_UDP_STR = "UDPStopNWait";

  public static final String GIOP_GIOP = "GIOP";  
  public static final String GIOP_LITE = "GIOP-Lite";  
  public static final String GIOP_MQC  = "MQC-IOP";  

  public static final String GIOP_GIOP_STR = "GIOP";  
  public static final String GIOP_LITE_STR = "GIOPLite";  
  public static final String GIOP_MQC_STR  = "MQCIOP";  

  public static final String GIOP_V10_STR  = "GIOP10";
  public static final String GIOP_V11_STR  = "GIOP11";
  public static final String GIOP_V12_STR  = "GIOP12";
  public static final String GIOP_V13_STR  = "GIOP13";
  
  public static final String NO_RESTRICTION = "(no restriction)";

  public static final int GIOP_HDR_SIZE = 12;  
  //size is not including the operation name but including 4 bytes for size of operation name    
  public static final int GIOP_REQ_SIZE = 24; 
  public static final int GIOP_REPLY_SIZE = 8;    
  public static final int GIOP_LITE_HDR_SIZE = 8;  
  //size is not including the operation name but including 4 bytes for size of operation name    
  public static final int GIOP_LITE_REQ_SIZE = 12; 
  public static final int GIOP_LITE_REPLY_SIZE = 5;    
  public static final int GIOP_MQC_HDR_SIZE = 10;  
  //size is not including the operation name but including 4 bytes for size of operation name    
  public final static byte MQC_SIZE_OF_HDR = 12;
  public final static byte MQC_SIZE_OF_REQ_HDR = 24; //+ op. length
  public final static byte MQC_SIZE_OF_REPLY_HDR = 8;
}
