/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

//include(`Config_macros.m4')

public class Timings
{
  static boolean dummy = true;
  final static int DT_MAX = 1500;
  
  public static void main(String[] args)
  {
    int[] dta = new int[DT_MAX];
    long  t, t0;
    int   dt;
    long  x, x2;
    float stdev;
    
    // Get the number of iterations
    //
    int iterations = 1000000;  // The default number of iterations
    String cntStr = System.getProperty("cnt");
    if (cntStr != null) {
      iterations = Integer.parseInt(cntStr);
    }
    System.out.println("Iteration 'cnt' set to " + iterations);

    int ssIterations = iterations;  // steady state iteration count

    long ssStartTime = 0;  // steady state start time
    long ssTotalTime = 0;  // steady state total time
    long startTime   = 0;  // start of "real" timing loop time
    long totalTime   = 0;  // total time for the "real" timing loop
    
    System.out.println("Ready to start the timing loop");


    // Time: loop

    ssStartTime = System.currentTimeMillis();
    for(int i = 0; i < ssIterations; i++) {
      // no DEBUG timing code here, save for the "real" timing loop
      // fooObj.bar(i);
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);
    
    // Output the intermediate steady state timing values
    float ssAvgTime = (float) ssTotalTime / ssIterations;
    System.out.println("*** N.State time (s):  " + (ssTotalTime)/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime));


    // Time: if ...

    ssStartTime = System.currentTimeMillis();
    for(int i = 0; i < ssIterations; i++) {
      if (i == -1) {
        dummy = false;
      }
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);
    
    // Output the intermediate steady state timing values
    ssAvgTime = (float) ssTotalTime / ssIterations;
    System.out.println("*** I.State time (s):  " + (ssTotalTime)/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime));


    // Time: freeMemory

    ssStartTime = System.currentTimeMillis();
    for(int i = 0; i < ssIterations; i++) {
      runtime.freeMemory();
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);
    
    // Output the intermediate steady state timing values
    ssAvgTime = (float) ssTotalTime / ssIterations;
    System.out.println("*** F.State time (s):  " + (ssTotalTime)/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime));


    // Time: currentTimeMillis

    ssStartTime = System.currentTimeMillis();
    t0 = System.currentTimeMillis();
    for(int i = 0; i < ssIterations; i++) {
      t = System.currentTimeMillis();
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);
    
    // Output the intermediate steady state timing values
    ssAvgTime = (float) ssTotalTime / ssIterations;
    System.out.println("*** Sys.cTM time (s):  " + (ssTotalTime)/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime) );


    // Time: currentTimeMillis w/ if test and division

    for (int i = 0;  i < DT_MAX;  i++) {
      dta[i] = 0;
    }
    ssStartTime = System.currentTimeMillis();
    t0 = System.currentTimeMillis();
    for(int i = 0; i < ssIterations; i++) {
      t = System.currentTimeMillis();
      dt = (int) (t - t0) / 1;
      t0 = t;
      if (dt < DT_MAX) {
        dta[dt]++;
      } else {
        dta[DT_MAX-1]++;
      }
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);
    x  = 0;
    x2 = 0;
    for (int i = 0;  i < DT_MAX; i++) {
      x  += dta[i] * i;
      x2 += dta[i] * i * i;
    }
    stdev = ((float) x2 - (float) (x * x)/ssIterations)/ssIterations;
    //float stdev = (float) Math.sqrt(stdev2);
    
    // Output the intermediate steady state timing values
    ssAvgTime = (float) ssTotalTime / ssIterations;
    System.out.println("*** Sys.cTM dt   (s):  " + (ssTotalTime)/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime) );
    System.out.println("    x: " + x + " x^2: " + x2 + 
                       " stdev^2: " + FormatFloat(stdev) );


    // Init the JNITimer

    JNITimer.init(2000000);


    // Time: JNITimer.currentTime

    ssStartTime = System.currentTimeMillis();
    for(int i = 0; i < ssIterations; i++) {
      //System.currentTimeMillis();
      JNITimer.currentTime();
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);

    ssAvgTime   = (float)  ssTotalTime / ssIterations;

    System.out.println("***  JNI.cT time (s): " + ssTotalTime/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime));


    // Time: JNITimer.currentTime w/ if test and division

    for (int i = 0;  i < DT_MAX;  i++) {
      dta[i] = 0;
    }
    ssStartTime = System.currentTimeMillis();
    t0 = JNITimer.currentTime();
    for(int i = 0; i < ssIterations; i++) {
      //t = System.currentTimeMillis();
      t = JNITimer.currentTime();
      dt = (int) (t - t0) / 1;
      t0 = t;
      if (dt < DT_MAX) {
        dta[dt]++;
      } else {
        dta[DT_MAX-1]++;
      }
    }
    ssTotalTime = (System.currentTimeMillis() - ssStartTime);
    x  = 0;
    x2 = 0;
    for (int i = 0;  i < DT_MAX; i++) {
      x  += dta[i] * i;
      x2 += dta[i] * i * i;
    }
    stdev = ((float) x2 - (float) (x * x)/ssIterations)/ssIterations;
    //float stdev = (float) Math.sqrt(stdev2);
    
    // Output the intermediate steady state timing values
    ssAvgTime = (float) ssTotalTime / ssIterations;
    System.out.println("***  JNI.cT dt   (s):  " + (ssTotalTime)/1000 +
                      " *** Average time (ms): " + FormatFloat(ssAvgTime) );
    System.out.println("    x: " + x + " x^2: " + x2 + 
                       " stdev^2: " + FormatFloat(stdev) );


    AllDone();
  }
  
  static void AllDone() {
    System.out.println("All Done!");
  }
  
  static String FormatFloat(float f) {
    long fint = (long) f;
    long ffra = (long) (10000 * (f - fint + 1));
    StringBuffer ffraStr = new StringBuffer( String.valueOf(ffra));
    ffraStr.setCharAt(0, '.');
    return String.valueOf(fint) + ffraStr;
  }

  static Runtime runtime = Runtime.getRuntime();
  
}
