/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

/**
 * @author Kim Christian Swenson
 */
public class GIOPHeader_1_0 extends GIOPHeader
{
   public GIOPHeader_1_0()
   {
      //Initialize defaults
      m_msgType = GIOPMessage.REQUEST_MSG;
      m_msgSize = 0;
      m_littleEndian = false;
      m_major = 1;
      m_minor = 0;
   }

   public void deMarshal( CDRInputStream in ) throws RuntimeException
   {
      boolean isGIOP = 
            in.readOctet() == GMAGIC &&
            in.readOctet() == IMAGIC &&
            in.readOctet() == OMAGIC &&
            in.readOctet() == PMAGIC;

      if ( ! isGIOP )
      {
         throw new RuntimeException("Not a GIOP header.");
      }

      boolean isMajorOne = in.readOctet() == m_major;
      boolean isMinorZero = in.readOctet() == m_minor;

      if ( ! isMajorOne || ! isMinorZero )
      {
         throw new RuntimeException("Wrong GIOP version, should be " + m_major
               + "." + m_minor );
      }

      m_littleEndian = in.readBoolean();

      in.setBigEndian( ! m_littleEndian );

      m_msgType = in.readOctet();

      m_msgSize = in.readUnsignedLong();
   }

   public void marshal( CDROutputStream out )
   {
      out.writeOctet( GMAGIC );
      out.writeOctet( IMAGIC );
      out.writeOctet( OMAGIC );
      out.writeOctet( PMAGIC );
      out.writeOctet( m_major );
      out.writeOctet( m_minor );
      out.writeBoolean( m_littleEndian );
      out.setBigEndian( ! m_littleEndian );
      out.writeOctet( m_msgType );
      out.writeUnsignedLong( m_msgSize );
   }

}
