/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.protocols.*;
import mqc.holders.*;
import mqc.util.*;
import java.io.*;
import java.util.*;
__CLDC__(
import javax.microedition.io.*;
)
__J2SE__(
import java.net.*;
)
__SECURITY__(
import mqc.security.GeneralSecurityException;
)
__SECURITY_MD__(
import mqc.security.DigestException;
)
__SECURITY_MAC__(
import mqc.security.ShortBufferException;
)

/**
 * Unreliable udp session handler.
 *
 * @author Thor Egil Skaug    security and optimization
 * @author Kevin E. Dorow     groupcomm
 * @author Eivind Naess       cldc
 * @author Tarana Damania     initial
 * @version 1.00
 */

public class S_UDPHandler extends S_UDPGenericHandler implements java.lang.Runnable
{
   private CDRInputStream m_in;
   private CDROutputStream m_out;

   public Thread m_activity = new Thread(this);
   __J2SE__(
   public S_UDPHandler(DatagramSocket clientSocket,String strClientName, int nClientPort, S_ORB orb)
   )
   __CLDC__(
   public S_UDPHandler(DatagramConnection clientSocket,String strClientURL, S_ORB orb)
   )
   {
      __J2SE__(
      super(clientSocket, strClientName, nClientPort, orb);
      )
      __CLDC__(
      super(clientSocket, strClientURL, orb);
      )

      m_out = new CDROutputStream( m_recvBuf.length, 0, false );
      m_in = new CDRInputStream( null, 0, false);

      //tes: this crashes on SAJE when done a second time (ie. two instances of this class)!!!
      m_activity.start();
   }

   public void run()
   {
      Marshalable msg;

      __SECURITY__(
      __SECURITY_MAC__(
      byte [] m_compMAC;
      byte [] m_recvMAC;
      )
      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      byte [] m_compMD;
      byte [] m_recvMD;
      )
      )
      )

      try
      {
         if(m_recvPacket == null)
         {
            __J2SE__(
            m_recvPacket = new DatagramPacket(m_recvBuf, m_recvBuf.length, InetAddress.getByName(m_strClientName), m_nClientPort);
            )
            __CLDC__(
            m_recvPacket = m_clientSocket.newDatagram(m_recvBuf, m_recvBuf.length);
            )
         }
      }
      __J2SE__(
      catch(UnknownHostException uhe)
      {
         uhe.printStackTrace();   
      }
      )
      catch(IOException ioe)
      {
         ioe.printStackTrace();  
      }

      int nLen = 0; //length of received message
      while(S_ORB.keepRunning)
      {
         msg = null;
         try
         {
            //receive the entire message
            m_clientSocket.receive(m_recvPacket);
            nLen = m_recvPacket.getLength(); 
         }
         catch(IOException ioe)
         {
            System.err.println("Failed to receive from client");
            ioe.printStackTrace();
            continue;
         }

         __VAR_SIZE__(
         boolean bigEndian = Message.isBigEndian( m_recvBuf );
         m_nHeaderSize = Message.getHdrSize(m_recvBuf);
         m_nMessageSize = Message.getMsgSize(m_recvBuf, bigEndian) + m_nHeaderSize;

         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         m_nMdOffset = m_nMessageSize;
         m_nMessageSize += ORB.msgDigestLen;
         )
         )
         __SECURITY_MAC__(
         m_nMacOffset = m_nMessageSize;
         m_nMessageSize += ORB.msgMacLen;
         )
         __SECURITY__(
         m_nPadOffset = m_nMessageSize;
         m_nMessageSize = m_orb.cipherDecrypt.getOutputSize(m_nMessageSize-m_nHeaderSize) + m_nHeaderSize;

         //variable size decrypt
         try
         {
            __DEBUG__(
            System.out.println("Varsize decrypt");
            System.out.println("encrypted: ");
            //mqc.util.msgPrinter.printHex(m_recvBuf,0,m_nMessageSize);
            )
            ORB.cipherDecrypt.doFinal(m_recvBuf, m_nHeaderSize, m_nMessageSize - m_nHeaderSize, m_recvBuf, m_nHeaderSize);
            __DEBUG__(
            System.out.println("unencrypted: ");
            //mqc.util.msgPrinter.printHex(m_recvBuf,0,m_nMessageSize);
            )
         }
         catch(GeneralSecurityException gse)
         {
            gse.printStackTrace();   
         }      
         )
         )

         __FIX_SIZE__(
         //in FIX_SIZE all the offsets are constant throughout the lifetime of this object
         __SECURITY__(
         try
         {
            ORB.cipherDecrypt.doFinal(m_recvBuf,0,nLen,m_recvBuf,0);
         }
         catch(GeneralSecurityException gse)
         {
            gse.printStackTrace();   
         }
         )
         )

         __SECURITY_MAC__(
         m_orb.msgMac.reset();
         m_orb.msgMac.update(m_recvBuf, 0, m_nMacOffset);
         m_compMAC = m_orb.msgMac.doFinal();
         m_recvMAC = new byte[m_orb.msgMacLen];
         System.arraycopy(m_recvBuf, m_nMacOffset, m_recvMAC, 0, m_orb.msgMacLen);
         if (!m_orb.msgMac.isEqual(m_compMAC, m_recvMAC))
         {
            System.err.println("***Invalid MAC");
            continue;
         }
         )

         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         m_orb.msgDigest.reset();
         m_orb.msgDigest.update(m_recvBuf, 0, m_nMdOffset);

         m_compMD = m_orb.msgDigest.digest();
         m_recvMD = new byte[m_orb.msgDigestLen];

         System.arraycopy(m_recvBuf, m_nMdOffset, m_recvMD, 0, m_orb.msgDigestLen);

         if (!m_orb.msgDigest.isEqual(m_compMD, m_recvMD))
         {
            System.err.println("***Invalid message digest");
            continue;
         }
         )
         )

         Request message = m_orb.findProtocolHandler(this, m_recvBuf);
         __DEBUG4__(//mqc.util.msgPrinter.printMessage(recvbuf, msgSize);)

         m_in.setBuffer( m_recvBuf );
         message.deMarshal( m_in );

         /*tes: THIS NEEDS A SERIOS REVIEW*/
         __GROUPCOM__(
         // make a copy of the data in the byte array
         byte[] actBuffer = new byte[length];
         System.arraycopy(buffer,0,actBuffer,0,length);
         try
         {
            String strBuffer = new String(actBuffer);
            __DEBUG__( System.err.println("Buffer2String is: " + strBuffer);)
            // send message using groupcom to replicants
            m_orb.group.sendMessage(strBuffer,"message","","");
         }
         catch (Exception e)
         {
            __DEBUG__( e.printStackTrace();)
         }
         )

         __TEMPSPACE__(
         // next check the message request id
         int intReqID = message.getRequestID();
         __DEBUG__( System.out.println("The received request ID is: " + intReqID);)
         // scan the buffer to see if this message has already been received
         Integer intCReqID = new Integer(intReqID);
         if(m_vMessageBuffer.contains(intCReqID))
         {
            __DEBUG__( System.out.println("Detected duplicate message with ID: " + intReqID);)
            // here we need to ignore the message completely
         }
         else
         {
            // here we need to insert the new id into the buffer
            __DEBUG__( System.out.println("Detected a new message to be processed with ID: " + intReqID);)
            m_vMessageBuffer.addElement(intCReqID);
            if(m_vMessageBuffer.size() > 50)
            {
               m_vMessageBuffer.removeElementAt(0);
            }
            msg = m_orb.serveRequest( message );
         }
         )
         __NOTEMPSPACE__(
         msg = m_orb.serveRequest( message );
         )        

         if ( msg != null )
         {
            // Marshal Reply message on stream
            msg.marshal( m_out );

            // Send Reply message
            send( m_out.getBuffer(), m_out.bytesStreamed() );
         }

      }

      m_orb.removeProtocolHandler(this);

      //hack to enforce the shutting down of the S_ORB
      if (!S_ORB.keepRunning)
      {
         S_ORB.shutDown(-1);
      }       

   }

  /**
   * Sends a message. If you enabled fixed size messages; all messages will be transmitted by
   * that length over the network. The other option is to enable variable length packets. As
   * indicated by the name the transport layer sends the actual packet length over the network.
   * The only difference is that under variable length packets the GIOPHeader is transmitted unencrypted
   * along the network wires. 
   *
   * Note that even if the member variables m_nMessageSize, m_nHeaderSize, + all the offset variables
   * are used here, there is no risk of any concurrency problems with run(), because this method is
   * actually a result of this::run()::orb::serveRequest(), so run() is blocking while this method
   * executes.
   *
   * @param byte[] The buffer to send
   * @param int The number of bytes to transmit
   * @return boolean true if data is sent successfully, false otherwise
   */   
   public synchronized boolean send(byte[] buffer, int msgSize)
   {    
      __VAR_SIZE__(
      m_nHeaderSize = Message.getHdrSize(buffer);
      m_nMessageSize = msgSize;

      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      m_nMdOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgDigestLen;
      )
      )

      __SECURITY_MAC__(
      m_nMacOffset = m_nMessageSize;
      m_nMessageSize += ORB.msgMacLen;
      )

      __SECURITY__(
      m_nPadOffset = m_nMessageSize;
      m_nMessageSize += m_orb.cipherDecrypt.getOutputSize(m_nMessageSize - m_nHeaderSize) + m_nHeaderSize;

      if (m_sendBuf == null || m_sendBuf.length < m_nMessageSize)
      {
         m_sendBuf = new byte [m_nMessageSize];
      }
      )
      )

      try
      {
         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         try
         {
            m_orb.msgDigest.reset();
            m_orb.msgDigest.update(buffer, 0, m_nMdOffset);
            m_orb.msgDigest.digest(buffer, m_nMdOffset, ORB.msgDigestLen);
         }
         catch(DigestException e)
         {
            e.printStackTrace();
         }
         )
         )
         __SECURITY_MAC__(
         try
         {
            m_orb.msgMac.reset();
            m_orb.msgMac.update(buffer, 0, m_nMacOffset);
            m_orb.msgMac.doFinal(buffer, m_nMacOffset);
         }
         catch(ShortBufferException sbe)
         {
            sbe.printStackTrace();
         }
         catch(IllegalStateException ise)
         {
            ise.printStackTrace();
         }
         )

         __SECURITY__(
         try
         {
            __FIX_SIZE__(
            //in fixedsize we encrypt the entire buffer
            ORB.cipherEncrypt.doFinal(buffer, 0, m_nPadOffset, m_sendBuf, 0);
            )
            __VAR_SIZE__(
            //in varsize we leave the header unencrypted
            System.arraycopy(buffer, 0, m_sendBuf, 0, m_nHeaderSize);
            ORB.cipherEncrypt.doFinal(buffer, m_nHeaderSize, m_nPadOffset - m_nHeaderSize, m_sendBuf, m_nHeaderSize);
            )
         }
         catch(GeneralSecurityException gse)
         {
            gse.printStackTrace();         
         }
         buffer = m_sendBuf;
         )   
      }
      catch(Exception e)
      {
         e.printStackTrace();
         return false;
      }

      try
      {
      if(m_sendPacket == null)
      {
         __J2SE__(
         m_sendPacket = new DatagramPacket(buffer, m_nMessageSize, InetAddress.getByName(m_strClientName), m_nClientPort);
         )
         __CLDC__(
         m_sendPacket = m_clientSocket.newDatagram(buffer, m_nMessageSize, m_strClientSessionURL);
         )
      }
      else
      {
         __J2SE__(
         m_sendPacket.setData(buffer);
         )
         __CLDC__(
         m_sendPacket.setData(buffer, 0, m_nMessageSize);
         )
      }
      }
      __J2SE__(
      catch (UnknownHostException uhe)
      {
         uhe.printStackTrace();
      }
      )
      catch(IOException ioe)
      {
         ioe.printStackTrace();   
      }

      try
      {
         __TEMPORAL__(
         for(int i=0;i<Config.temporal;i++)
         {
         )
         //send the datagram
         m_clientSocket.send(m_sendPacket);       
         __TEMPORAL__(
         }
         )
      }
      catch(IOException ioe)
      {
         ioe.printStackTrace();
         return false;
      }
      return true;
   }
}
