/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.ior.*;
import mqc.holders.*;
import mqc.protocols.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.*;
__J2SE__(
import java.net.*;
)
__CLDC__(
import javax.microedition.io.Connection;
import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
// Use the MemoryEfficientConnector (add at a future date)
//import com.ajile.microedition.io.MemoryEfficientConnector;
)

__SECURITY__(
import mqc.security.GeneralSecurityException;
)
__SECURITY_MD__(
import mqc.security.DigestException;
)
__SECURITY_MAC__(
import mqc.security.ShortBufferException;
)

/**
* TCP transport class
*
* @author Olav Haugan
* @author David McKinnon
*/
public class TCPTransport extends Transport
{
   private String host;
   private int port;
   __J2SE__(
   private Socket socket;
   )
   __CLDC__(
   private String sockURL;
   )

   private byte [] m_recvBuf = null;      //used in receive
   byte[] m_sendBuf = null;                    //used in send()

   private DataOutputStream out;                //socket.getOutputStream()
   private DataInputStream in;          //socekt.getInputStream()
   private int protocol;                                //???
   private int msgSize=0;                               //Sets the size of the message
   __SECURITY__(
   private int padOffset=0;                     //sets the padding size for encrypted msg.
   __VAR_SIZE__(
   byte [] pbuffer=null;                                //used to store the decrypted result in receive
   )
   )
   __SECURITY_MAC__(
   private int macOffset=0;                     //sets the offset for mac
   )
   __SECURITY_MD__(
   __SECURITY_MD_NOT_HMAC__(
   private int mdOffset=0;                              //sets the offset for md
   ))

   __TEMPSPACE__(
   int lastRequestID = -1;
   )

   /**
    * Constructor. Initializes variables.
    */
   /*
   public TCPTransport()
   {
      super();
      host = null;
      port = 0;

      __J2SE__(
      socket = null;
      )
      __CLDC__(
      sockURL = null;
      )

      m_in = new CDRInputStream();
      __VAR_SIZE__(m_out = new CDROutputStream(0, 0, false);)
      __FIX_SIZE__(m_out = new CDROutputStream(mqc.Config.intPacketSize, 0, false);)
   }
   */

   /**
    * Constructor. Initializes variables. Connects to the server.
    *
    * @param mqc.Delegate The delegate object
    */
   public TCPTransport(mqc.Delegate delegate)
   {
      super();
      Profile profile = delegate.ior.getProfile();
      protocol = delegate.protocol;
      TaggedProfile tagProfile = null;
      if ( profile.getTaggedProfile(TaggedProfile.IIOP) != null)
      {
         tagProfile = profile.getTaggedProfile(TaggedProfile.IIOP);
      }
      else if (profile.getTaggedProfile(TaggedProfile.IIOPLITE) != null)
      {
         tagProfile = profile.getTaggedProfile(TaggedProfile.IIOPLITE);
      }
      else if (profile.getTaggedProfile(TaggedProfile.MQCIOP) != null)
      {
         tagProfile = profile.getTaggedProfile(TaggedProfile.MQCIOP);
      }
      else
      {
         System.err.println ("**Error: no transport layer found");
      }
      host = tagProfile.host;
      port = tagProfile.port;
      __CLDC__(
      sockURL = "socket://" + host + ":" + port;
      )

      init();
   }

   /**
    * Initializes a connection.
    * We'll connect to the server here.
    *
    * @return boolean true if socket created successfully, false otherwise
    */
   public boolean init()
   {
      try
      {
         __J2SE__(
         socket =  new Socket(host, port);
         socket.setTcpNoDelay(false);
         out = new DataOutputStream(socket.getOutputStream());
         in = new DataInputStream(socket.getInputStream());
         )
         __CLDC__(
         StreamConnection sock = (StreamConnection) Connector.open(sockURL);
         // use the new MemoryEfficientConnector (Add at a future date)
         // StreamConnection sock = (StreamConnection)
         //                         MemoryEfficientConnector.open("socket",
         //                                                        ipNum, portNum);
         //sock.setTcpNoDelay(false);
         out = (DataOutputStream) sock.openDataOutputStream();
         in = (DataInputStream) sock.openDataInputStream();
         )
      }
      __J2SE__(
      catch(UnknownHostException e)
      {
         System.err.println("Unknown host");
         return false;
      }
      )
      __CLDC__(
      catch(ConnectionNotFoundException e)
      {
         System.err.println("Connection Not Found");
         return false;
      }
      )
      catch(IOException e)
      {
         System.err.println("Error creating socket");
         System.err.println(e);
         return false;
      }

      __FIX_SIZE__(
      /* FIX_SIZE BEGIN */
      //Since this is fixed messages then we can asumme a statical initialized buffer and
      //constants! 
      msgSize = Config.intPacketSize;

      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(    // do not add MD len if the MD is for
      mdOffset = msgSize;
      msgSize += ORB.msgDigestLen;  // an HMAC (ie, no double counting)
      ))

      __SECURITY_MAC__(
      macOffset = msgSize;
      msgSize += ORB.msgMacLen;
      )

      __SECURITY__(
      padOffset = msgSize; //Set the padding offset (no double padding)
      msgSize = ORB.cipherDecrypt.getOutputSize(msgSize);
      )
      m_recvBuf = new byte [ msgSize ];
      m_sendBuf = new byte [ msgSize ];
      /* FIX_SIZE END */
      )

      m_in = new CDRInputStream();
      m_out = new CDROutputStream(msgSize, 0, false);

      return true;
   }

   /**
    * Sends a message - The buffer that is passed to this function is assumed to have enough space
    * that includes eventual security settings (HMAC|MD|PADDING) However, the length is the last 
    * possition that is written to (EOM).
    * 
    * When using variable length packet the header is sent without encryption, when using fixed
    * length packets then the buffer is totally encrypted.
    *
    * @param buffer   The buffer to send 
    * @param length   The number of bytes to transmit
    * @return boolean true if data is sent successfully, false otherwise
    */
   public boolean send(byte[] buffer, int length)
   {
      __DEBUG__(
      System.err.println("TCPTransport::send -- buffer length: " +
                          length + " (" + Config.intPacketSize + ")");
      )

      __VAR_SIZE__(
      /* VAR_SIZE BEGIN */
      // compute the size of the variable length package to send

      int msgHdrSize = Message.getHdrSize(buffer);
      msgSize = length;

      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(     // do not add MD len if the MD is for
      mdOffset = msgSize;
      msgSize += ORB.msgDigestLen;  // an HMAC (ie, no double counting)
      ))

      __SECURITY_MAC__(
      macOffset = msgSize;
      msgSize += ORB.msgMacLen;
      )

      __SECURITY__(
      // This looks little wierd! - We only want to encrypt everything but the header 
      // and must ensure there is enough buffer. cipher.EncryptDoFinal seem to add another 
      // empty block size of bytes to the message we want to encrypt. msgSize remains with the
      // value, but another block size is added to the output buffer.

      padOffset = msgSize; //No dobble padding...
      //Now get the padded size of the message we want to encrypt
      msgSize = ORB.cipherDecrypt.getOutputSize(msgSize-msgHdrSize) + msgHdrSize;
      )

      //Resize the buffer if needed to fit our purpose!
      if (m_sendBuf == null || m_sendBuf.length < msgSize)
      {
         m_sendBuf = new byte [msgSize];
      }
      /* VAR_SIZE END */
      ) 

      __DEBUG4__(
      //mqc.util.msgPrinter.printMessage(buffer, length);
      )

      try
      {
         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         try
         {
                                //Add in the MD value at mdOffset!
            ORB.msgDigest.reset();
            ORB.msgDigest.update(buffer, 0, mdOffset);
            ORB.msgDigest.digest(buffer, mdOffset, ORB.msgDigestLen);
         }
         catch (DigestException de)
         {
            de.printStackTrace();
         }
         ))

         __SECURITY_MAC__(
         try
         {
                                //Add in the MAC value at macOffset
            ORB.msgMac.reset();
            ORB.msgMac.update(buffer, 0, macOffset);
            ORB.msgMac.doFinal(buffer, macOffset);
         }
         catch (ShortBufferException sbe)
         {
            sbe.printStackTrace();
         }
         catch (IllegalStateException ise)
         {
            ise.printStackTrace();
         }
         )

         __SECURITY__(
         try
         {
            __FIX_SIZE__(
            //Encrypt the whole message -- input buf - start - unpadded length - outputbuf - offset
            ORB.cipherEncrypt.doFinal(buffer, 0, padOffset, m_sendBuf, 0);
            )
            __VAR_SIZE__(
            //encrypt the message body
            System.arraycopy(buffer, 0, m_sendBuf, 0, msgHdrSize);
            /* input buf, input offset, num unpadded bytes, output buf, output offset - returns the number */
            /* of bytes written to the sendpacket */
            ORB.cipherEncrypt.doFinal(buffer, msgHdrSize, padOffset-msgHdrSize, m_sendBuf, msgHdrSize);
            )
         }
         catch (GeneralSecurityException gse)
         {
            gse.printStackTrace();
         }
         //Reusing variable names -- we don't want to use arraycopy
         buffer = m_sendBuf;
         )

         __TEMPORAL__(
         // Use the temporal variable in the Config file to determine what the
         // level of temporal redundancy is
         for(int i=0;i<Config.temporal;i++)
         {
         )
            // send the fixed length packet
            __DEBUG5__(//mqc.util.msgPrinter.printHex(buffer, 0, buffer.length);)
            out.write(buffer, 0, msgSize);
            __DEBUG__(System.err.println("TCPTransport.send(): buffer.length = " + buffer.length);)
         __TEMPORAL__(
         }
         )
         __DEBUG__( System.err.println("Client bytes sent: " + msgSize);)
      }
      catch(IOException e)
      {
         e.printStackTrace();
         return false;
      }
      return true;
   }

   /**
    * Waits for a message and receive it.
    *
    * @param msg       The message sent
    * @return byte[]   The buffer received
    */
   public boolean waitForReply( mqc.protocols.Reply rep )
   {
      __DEBUG5__(System.err.println("TCPTransport.waitForReply()");)
      byte [] buffer = null;
      __VAR_SIZE__(
      msgSize=0;
      /* VAR_SIZE SIZE */
      int msgHdrSize = rep.getHdrSize();
      if (m_recvBuf == null || m_recvBuf.length < msgHdrSize)
      {
         m_recvBuf = null;
         m_recvBuf = new byte [msgHdrSize];
      }
      try 
      {
         //Read the first bytes of the header
         in.readFully(m_recvBuf, 0, msgHdrSize );

         // whether reply message Big Endian
         boolean bigEndian = Message.isBigEndian( m_recvBuf );

         //resolve the number of bytes in the message buffer
         msgSize = Message.getMsgSize(m_recvBuf, bigEndian) + msgHdrSize;
         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(    // do not add MD len if the MD is for
         mdOffset = msgSize;
         msgSize += ORB.msgDigestLen;  // an HMAC (ie, no double counting)
         ))

         __SECURITY_MAC__(
         macOffset = msgSize;
         msgSize += ORB.msgMacLen;
         )

         __SECURITY__(
         //Get the block size!
         padOffset = msgSize;
         msgSize = ORB.cipherDecrypt.getOutputSize(msgSize-msgHdrSize)+msgHdrSize;
         )
         if (m_recvBuf.length < msgSize)
         {
            byte [] btmp = new byte [msgSize];
            System.arraycopy(m_recvBuf, 0, btmp, 0, msgHdrSize);
            m_recvBuf = null; //Helps with GC?
            m_recvBuf = btmp;
         }

         // Read rest of message
         in.readFully(m_recvBuf, msgHdrSize, msgSize-msgHdrSize);
         __DEBUG5__(//mqc.util.msgPrinter.printHex(m_recvBuf, 0, msgSize);)
         __SECURITY__(
         //Decrypt takes away padding - no need to have an external buffer for that -- reuse!
         ORB.cipherDecrypt.doFinal(m_recvBuf, msgHdrSize, msgSize-msgHdrSize, m_recvBuf, msgHdrSize);
         )
         buffer = m_recvBuf;
      }
      __SECURITY__(
      catch (GeneralSecurityException gse)
      {
         gse.printStackTrace();
         return false;
      }
      )
      catch (Exception e) 
      {
         System.err.println("**Error: " + e.getMessage());
         return false;
      }
      /* VAR_SIZE END */
      )

      __FIX_SIZE__(
      /* FIX_SIZE START */
      try
      {
         in.readFully(m_recvBuf, 0, msgSize ); //Read the whole fixed message
         __SECURITY__(
         ORB.cipherDecrypt.doFinal(m_recvBuf, 0, msgSize, m_recvBuf, 0);
         )
      }

      __SECURITY__(
      catch (GeneralSecurityException gse)
      {
         gse.printStackTrace();
         return false;
      }
      )
      catch (Exception e)
      {
         System.err.println("**Error: " + e.getMessage());
         return false;
      }
      /* FIX_SIZE END */
      )

      __SECURITY_MAC__(
      ORB.msgMac.reset();
      ORB.msgMac.update(m_recvBuf, 0, macOffset);

      byte [] compMAC = ORB.msgMac.doFinal();
      byte [] recvMAC = new byte[ORB.msgMacLen];

      System.arraycopy( m_recvBuf, macOffset, recvMAC, 0, ORB.msgMacLen);

      if (!ORB.msgMac.isEqual(compMAC, recvMAC))
      {
         System.err.println("MAC was NOT okay");
         return false;
      }
      )
      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(
      ORB.msgDigest.reset();
      ORB.msgDigest.update(m_recvBuf, 0, mdOffset);

      byte [] compMD = ORB.msgDigest.digest();
      byte [] recvMD = new byte[ORB.msgDigestLen];

      System.arraycopy(m_recvBuf, mdOffset, recvMD, 0, ORB.msgDigestLen);

      if (!ORB.msgDigest.isEqual(compMD, recvMD))
      {
         System.err.println("Message Digest was NOT okay");
         System.exit(-1);
      }
      )
      )

      __DEBUG4__(//mqc.util.msgPrinter.printMessage(m_recvBuf, m_recvBuf.length);)

      // Tell CDRInputStream to use m_recvBuf
      m_in.setBuffer( m_recvBuf );

      // DeMarshal message into reply
      rep.deMarshal( m_in );

      __TEMPORAL__(
      int tmpID = rep.getRequestID();
      if ( tmpID > lastRequestID )
      {
         lastRequestID = tmpID;
      }
      else
      {
         // Message is a duplicate
         __DEBUG5__(System.err.println( "TCPTransport.waitForReply(): skipped duplicate with ID " + tmpID );)
         return false;
      }
      )

      return true;
   }

   __TEMPORAL__(
   public int skipReply( Reply rep )
   {
      int nBytes = 0;
      __VAR_SIZE__(
      /*
       * Kim Christian Swenson's ideas on how to implement:
       * Request and Reply interfaces in the mqc.protocols package needs 
       * to implement the Cloneable interface with all it's implications.
       * This method must be called with a clone of the currently used Reply 
       * message. waitForReply should be called from within this method body
       * using the cloned reply message. Now since we are working with a 
       * clone, the original good/correct Reply message will not be 
       * tampered with. A similar implementation is needed on the Server side.
       */
      throw new RuntimeException( "NOT IMPLEMENTED: Temporal redundancy with variable packet length" );
      )
      __FIX_SIZE__(
      try
      {
         while ( in.available() >= msgSize )
         {
            // Skip fixed packet length while more than or equal to
            // one entire packet has been buffered in socket
            nBytes += in.skipBytes( msgSize );
            __DEBUG5__(System.err.println( "TCPTransport.skipReply(): skipped " + nBytes + " bytes." );)
         }
      }
      catch ( Exception e )
      {
         __DEBUG__(e.printStackTrace();)
      }
      )
      return nBytes;
   }
   )
}
